==> unixODBC-1.8.8.tar.gz: unixODBC-1.8.8/sqi/sqiSpecialColumns.c <==
#include "sqi.h"

/*
 * This does the same thing as sqiPrimaryKeys() right now... it
 * only differs in that it creates a result set with a diff struct.
 *
 * The idea is that in the future there will be a special column
 * called _ROWID which will be, when it is present, a date-time
 * stamp which, secretly, uniquely identifies each row.
 *
 */

int sqiSpecialColumns( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
	HSQITABLE	hTableRoot;
	int			nCol;

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( pszTableName );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pszTableName );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * BUILD RESULTS
	 ************************/
	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiSpecialColumns" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiSpecialColumns", "SCOPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiSpecialColumns", "COLUMN_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiSpecialColumns", "DATA_TYPE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiSpecialColumns", "TYPE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiSpecialColumns", "COLUMN_SIZE" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiSpecialColumns", "BUFFER_LENGTH" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 6, "system", "sqiSpecialColumns", "DECIMAL_DIGITS" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 7, "system", "sqiSpecialColumns", "PSEUDO_COLUMN" ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	hColHdrs = hStmt->hTableResult->hColHdrs;
	nCol = 0;
	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pColumn	= (HSQICOLUMN)lstGet( hColHdrs );
		if ( pColumn->bPimaryKey )
		{
			nCol++;
			pData		= malloc( sizeof(char*) * 9 );

			sprintf( szBuffer, "%d", SQL_SCOPE_SESSION );
			pData[0]	= (char *)strdup( szBuffer );
			pData[1]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );

			sprintf( szBuffer, "%d", pColumn->nSQL_DESC_TYPE );
			pData[2]	= (char *)strdup( szBuffer );
			pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_TYPE_NAME );

			sprintf( szBuffer, "%d", pColumn->nSQL_DESC_LENGTH );
			pData[4]	= (char *)strdup( szBuffer );
			pData[5]	= (char *)strdup( szBuffer );
			pData[6]	= (char *)strdup( "" );

			sprintf( szBuffer, "%d", SQL_PC_NOT_PSEUDO );
			pData[7]	= (char *)strdup( szBuffer );
	
			pData[8]	= NULL; /* data terminator... see _sqiFreeDataRow */
	
			lstAppend( hStmt->hTableResult->hRows, pData );
		}

		lstNext( hColHdrs );
	}

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}

==> unixODBC-1.8.13.tar.gz: unixODBC-1.8.13/sqi/sqiSpecialColumns.c <==
#include "sqi.h"

/*
 * This does the same thing as sqiPrimaryKeys() right now... it
 * only differs in that it creates a result set with a diff struct.
 *
 * The idea is that in the future there will be a special column
 * called _ROWID which will be, when it is present, a date-time
 * stamp which, secretly, uniquely identifies each row.
 *
 */

int sqiSpecialColumns( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
	HSQITABLE	hTableRoot;
	int			nCol;

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pszTableName );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pszTableName );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * BUILD RESULTS
	 ************************/
	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiSpecialColumns" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiSpecialColumns", "SCOPE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiSpecialColumns", "COLUMN_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiSpecialColumns", "DATA_TYPE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiSpecialColumns", "TYPE_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiSpecialColumns", "COLUMN_SIZE", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiSpecialColumns", "BUFFER_LENGTH", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 6, "system", "sqiSpecialColumns", "DECIMAL_DIGITS", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 7, "system", "sqiSpecialColumns", "PSEUDO_COLUMN", SQL_VARCHAR, 50 ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	hColHdrs = hStmt->hTableResult->hColHdrs;
	nCol = 0;
	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pColumn	= (HSQICOLUMN)lstGet( hColHdrs );
		if ( pColumn->bPimaryKey )
		{
			nCol++;
			pData		= malloc( sizeof(char*) * 9 );

			sprintf( szBuffer, "%d", SQL_SCOPE_SESSION );
			pData[0]	= (char *)strdup( szBuffer );
			pData[1]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );

			sprintf( szBuffer, "%d", pColumn->nSQL_DESC_TYPE );
			pData[2]	= (char *)strdup( szBuffer );
			pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_TYPE_NAME );

			sprintf( szBuffer, "%d", pColumn->nSQL_DESC_LENGTH );
			pData[4]	= (char *)strdup( szBuffer );
			pData[5]	= (char *)strdup( szBuffer );
			pData[6]	= (char *)strdup( "" );

			sprintf( szBuffer, "%d", SQL_PC_NOT_PSEUDO );
			pData[7]	= (char *)strdup( szBuffer );
	
			pData[8]	= NULL; /* data terminator... see _sqiFreeDataRow */
	
			lstAppend( hStmt->hTableResult->hRows, pData );
		}

		lstNext( hColHdrs );
	}

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}
